AMLEmode <- function(ABCsam, hh)
{

# This function approximates the mode of a multivariate empirical distribution by means of:

# 1. the sample mean
# 2. the product of the maxima of the univariate kernel densities estimated using the marginals
# 3. the Blurring Mean shift algorithm
# 4. the maximum of the multivariate kernel density
# 5. the maximum of the product of the univariate kd
#
# Input:
# ABCsam: (m x q) matrix, ABC sample
# hh: scalar >0,  bandwidth parameter
#
# Output:
#
# res, a list containing the 5 AMLE estimators
#
# Usage:
# hh = 0.9
# res <- AMLEmode(ABCsam,hh)
#
# Authors: Marco Bee, Roberto Benedetti, Giuseppe Espa
# This version: August 2016.

require(ks)
require(MeanShift)
N = nrow(ABCsam) # ABC sample size
qq = ncol(ABCsam)

# 1. sample mean

AMLE1 = colMeans(ABCsam)

# 2. univariate kd

AMLE2 = rep(0,qq)
for (i in 1:qq)
{
    vettore = ABCsam[,i]
    bw = hscv(vettore)
    f_eps = kde(vettore,bw,eval.points=vettore)
    AMLE2[i] = vettore[which.max(f_eps$estimate)]
}

# 3. Blurring Mean shift algorithm

res = bmsClustering(t(ABCsam),hh)
AMLE3 = res$components

# 4. multivariate kde

bw = Hscv(ABCsam)
f_eps = kde(ABCsam,bw,eval.points=ABCsam)
AMLE4 = ABCsam[which.max(f_eps$estimate),]

# 5. maximum of the product of the univariate kernel densities

f_eps_g = matrix(0,N,qq)
for (i in 1:qq)
{
    vettore = ABCsam[,i]
    bw = hscv(vettore)
    f_eps = kde(vettore,bw,eval.points=vettore)
    f_eps_g[,i] = f_eps$estimate
}
F_eps = apply(f_eps_g,1,prod)
AMLE5 = ABCsam[which.max(F_eps),]
res = list(SampleMean = AMLE1, UnivariateKD = AMLE2, BlurringMeanShift = AMLE3[,1], MultivariateKD = AMLE4, ProductUnivariateKD = AMLE5)
}