rBing <- function(nreps,b,A,M)

# This function simulates the q-dimensional Bingham distribution by means of the acceptance-rejection algorithm
# developed by Kent, J. T., Ganeiber, A. M., and Mardia, K. V. (2013). "A new method to simulate the
# Bingham and related distributions in directional data analysis with applications." 
# http://arxiv.org/abs/1310.8110.
#
# Input: 
#
# nreps: number of simulated random vectors
# b: tuning constant to be used in the simulation of the Bingham distribution. Typically set to 1.
# A: (q x q) matrix of parameters of the Bingham distribution
# M: constant to be used in the acceptance-rejection algorithm that simulated the Bingham distribution. It is 
# convenient to set M=sup{f/x)/g(x)}, where f(x) is the unnormalized Bingham density and g(x) is the 
# unnormalized angular central Gaussian density
# 
# Output:
#
# y: (nreps x q) matrix of nreps simulated vectors of length q
#
# Usage: 
# nreps <- 100
# b <- 1
# lambda_n <- c(.588,.421,0)
# A <- diag(lambda_n)
# ysim <- rBing(nreps,b,A,M)
#
# Authors: Marco Bee, Roberto Benedetti, Giuseppe Espa
# This version: August 2016.

{
source('c:\\marco\\dropbox\\R\\ACG.r')
require(emulator)
qq = nrow(A)
Psinv = diag(qq) + (2/b) * A
Psi = solve(Psinv)
y = matrix(0,1,qq)
n_obs = floor(nreps*M)
while (nrow(y)<(nreps+1)) 
{
  u = runif(n_obs) # Robert-Casella Monte Carlo R, p. 54
  x = ACG(n_obs,Psi)
  f = exp(-quad.diag(A,t(x)))
  g = quad.diag(Psinv,t(x))^(-qq/2)
  indici = f>u*g*M
  y = rbind(y,x[indici,])
}
y = y[2:(nreps+1),]
y
}